// src/backend/observedSubject.hh - Copyright 2005, 2006, University
//               of Padova, dept. of Pure and Applied
//               Mathematics
//
// This file is part of SGPEMv2.
//
// This is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// SGPEMv2 is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with SGPEMv2; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

#ifndef OBSERVEDSUBJECT_HH
#define OBSERVEDSUBJECT_HH 1

#include "config.h"

#include <vector>
#include <algorithm>

#include "../observer.hh"

namespace sgpem
{
  /**
     Abstract class which represents an observed entity who calls Update() in all Observer objects.
     See "Observer Pattern" for more information.
  */
  class ObservedSubject;
  
  class SG_DLLEXPORT ObservedSubject
  {
  public:
    virtual ~ObservedSubject() =0;
    
    void notify();
    void attach(sgpem::Observer*);
    bool detach(sgpem::Observer*);
    
  private:
    std::vector<Observer*> _attached;			
  };

} //~ namespace sgpem


#endif

