// src/backend/pyloader/python_policy.hh - Copyright 2005, 2006, University
//               of Padova, dept. of Pure and Applied
//               Mathematics
//
// This file is part of SGPEMv2.
//
// This is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// SGPEMv2 is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with SGPEMv2; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

#ifndef PYTHON_POLICY_HH
#define PYTHON_POLICY_HH 1

#include "config.h"

#include <Python.h>
#include <glibmm/ustring.h>

#include <iostream>

#include "../policy.hh"
#include "../user_interrupt_exception.hh"

namespace sgpem
{
  class PythonPolicy;
  class PythonPolicyManager;
  class UserInterruptException;
	
  class SG_DLLEXPORT PythonPolicy : public Policy
  {
  public:  
    //only PythonPolicyManager can create a PythonPolicy object
    friend class PythonPolicyManager;
	
    virtual ~PythonPolicy();
	
    void			configure() throw(UserInterruptException);
    void 			sort_queue(Scheduler::event) const throw(UserInterruptException);
    Glib::ustring 	        get_description() const;
    bool 			is_pre_emptive() const throw(UserInterruptException);
    int                         get_time_slice() const throw(UserInterruptException);

    // Singing with Caipha's voice: "must die, must die, this method must die!"
    void 			set_time_slice(const int&);
    			
  private:
    PythonPolicy(const char* name);
    PythonPolicy(const PythonPolicy&);
    PythonPolicy& operator=(const PythonPolicy&);

    void wait_unlock() const throw(UserInterruptException);

    PyObject* _adapter;
    PyObject* _adapter_dict;
    PyObject* _lock;

    Glib::ustring _name;
  };

}

#endif

