// src/backend/pyloader/python_policy_manager.cc - Copyright 2005, 2006, University
//               of Padova, dept. of Pure and Applied
//               Mathematics
//
// This file is part of SGPEMv2.
//
// This is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// SGPEMv2 is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with SGPEMv2; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

#include "python_policy_manager.hh"

#include <Python.h>
#include <glibmm/timer.h>

#include <string>
#include <cassert>
#include <iostream>
#include <unistd.h>
using namespace sgpem;

//static object
PythonPolicyManager* PythonPolicyManager::_instance = NULL;

PythonPolicyManager::PythonPolicyManager()
  : _initialized(false)
{
  PyEval_InitThreads();
}

PythonPolicyManager* const
PythonPolicyManager::get_instance()
{
  if(!_instance)
    _instance = new PythonPolicyManager();
  return _instance;
}

Policy&
PythonPolicyManager::get_policy()
{
  // FIXME : assumes that _python_policy is always != NULL!
    return *_python_policy;
}

void
PythonPolicyManager::init()
{
    if(_initialized) {
	_python_policy = std::auto_ptr<PythonPolicy>(0);
	Py_Finalize();
    }
  
  Py_Initialize();
  _initialized = true;
  
  // The following line is ugly, but necessary if we use 
  // non-standard installation directories. Theoretically,
  // it should be up to the user to set correct 
  // environment variables. 
  // FIXME: find better way to achieve this.
  
  PyRun_SimpleString("import sys\n"
		     "sys.path[:0] = [ '" MODDIR "', '" PYCDIR "' ]\n");

  // Okay, here we go.
  // Black magic at work. 

  // FIXME : Hardcoded policy name
  char* policy_name = "fcfs";
  //char* policy_name = "sjf";
  _python_policy = std::auto_ptr<PythonPolicy>(new PythonPolicy(policy_name));
}
